mutable struct MDFileStream
    const io::IO
    const filename::String
    linenumber::Int
    
    MDFileStream(io::IO, filename::String) = new(io, filename, 1)
end

function Base.readline(stream::MDFileStream)::String
    if !eof(stream.io)
        stream.linenumber += 1
    end
    readline(stream.io; keep=false) # fix GZipStream defaulting keep to true
end

const chunksize = 1024 * 1024

function readlines(stream::MDFileStream, n::Int)::Tuple{Vector{UInt8}, Int}
    v = UInt8[]
    nr = readlines!(stream, v, n)
    v, nr
end

function readlines!(stream::MDFileStream, b::Vector{UInt8}, n::Int)::Int
    initial_n::Int = n
    lastpos::Int = position(stream.io)
    it = 0
    
    while !eof(stream.io) && n > 0
        l = it + 1
        resize!(b, it + chunksize)
        s = readbytes!(stream.io, (@view b[l:it + chunksize]), chunksize)::Int
        it += s
        for i in l:it
            if b[i] == 0x0A
                n -= 1
                stream.linenumber += 1
                if n == 0
                    it = i
                    break
                end
            end
        end
    end
    
    seek(stream.io, lastpos + it)
    resize!(b, it)
    if it > 0 && b[it] != 0x0A
        n -= 1
    end
    initial_n - n
end

function readuntil(stream::MDFileStream, delim::AbstractChar)
    s = readuntil(stream.io, delim)
    stream.linenumber += count('\n', s)
    if delim == '\n'
        stream.linenumber += 1
    end
    s
end

function Base.eof(stream::MDFileStream)::Bool
    eof(stream.io)
end

function linenumber(stream::MDFileStream)::Int
    stream.linenumber
end

function filename(stream::MDFileStream)::String
    stream.filename
end
