struct TableParsingError <: Exception
    row::Int
    col::Int
    celltype::Type
end

Base.showerror(io::IO, e::TableParsingError) = print(io, "Parsing table failed at row $(e.row), column $(e.col), expected cell type $(e.celltype)")

function readtable(v::Vector{UInt8}; kwargs...)::DataFrame
    df = CSV.read(v, DataFrame; validate=false, silencewarnings=true, delim=' ', ignorerepeated=true, kwargs...)
    select!(df, kwargs[:header])
    checkmissing(df)
    df
end

function checkmissing(df::DataFrame)
    for (i, col) in enumerate(eachcol(df))
        if Missing <: eltype(col)
            for (j, cell) in enumerate(col)
                if ismissing(cell)
                    throw(TableParsingError(j, i, nonmissingtype(eltype(col))))
                end
            end
        end
    end
end
